
#include <zephyr.h>
#include <misc/reboot.h>
#include <device.h>
#include <string.h>
#include <flash.h>
#include <nvs/nvs.h>

#include <gps_.h>
#include <led.h>
#include <button.h>
#include <mqtt_.h>

struct nvs_fs fs;

int nvs_setup(){
  int retval = 0;
  struct flash_pages_info info;

  fs.offset = DT_FLASH_AREA_STORAGE_OFFSET;
//  fs.offset = PM_MCUBOOT_STORAGE_ADDRESS;
  retval = flash_get_page_info_by_offs(device_get_binding(DT_FLASH_DEV_NAME), fs.offset, &info);
  if(retval){
    printk("Unable to get page info");
    return retval;
  }

  fs.sector_size = info.size;
  fs.sector_count = 3U;

  retval = nvs_init(&fs, DT_FLASH_DEV_NAME);
  if(retval){
    printk("Flash Init failed\n");
    return retval;
  }
  return 0;
}

void test_gps(){

    spoof(false);

    led_setup();
    button_setup();

// Storage
    // Set up certificates before nvs or else permission errors
    provision_certificates();

    nvs_setup();
    printk("\nNVS set up\n");
//---

    k_sleep(1000);

// MQTT
    int err;
    struct mqtt_client client;
    struct sockaddr_storage broker;
    struct pollfd fds;

    modem_configure();
    client_init(&client, &broker, CONFIG_MQTT_BROKER_HOSTNAME);
    aws_fota_init(&client, CONFIG_APP_VERSION, aws_fota_cb_handler);
    err = mqtt_connect(&client);
    if (err != 0) {
            printk("ERROR: mqtt_connect %d\n", err);
            return;
    }
    fds_init(&client, &fds);
    boot_write_img_confirmed();
//---

// GPS
    GPS_Init();
    printk("\nGPS Initialized");
    GPS_Enable();
    printk("\nGPS Enabled");
//---

    s64_t loop_timer = k_uptime_get();
    s64_t publish_timer = k_uptime_get();
    s64_t live_timer = k_uptime_get();

    GPS_Logging_Off();

// Defs
    bool subbed = false;
    char strprint[100];
//    int num_of_locs;
//    struct coordinate_data coords_data[25];
//    char bigprint[100*25];

//---

    while(1){

// GPS Loop
        if(detect_press(2)){
            if(GPS_Is_Logging()){
                GPS_Logging_Off();
            }else{
                GPS_Logging_On();
            }
        }

        if(detect_press(1)){
            while(!detect_press(1));
        }

        GPS_State_Machine(&fs);
//---
// MQTT Loop

        err = poll_routine(&client, &fds);
        if(err) break;

        
        if(k_uptime_get()-live_timer >= 1000*(CONFIG_MQTT_KEEPALIVE/2)){
            if(keep_mqtt_live(&client) != 0){
            }
            live_timer = k_uptime_get();
        }

        if(!subbed && read_client_state(&client) == 6){
              subscribe(&client, "pubtopic/test");
              subbed = true;
        };


        if(k_uptime_get()-publish_timer >= 5000 && subbed){
              publish_timer = k_uptime_get();  

              int loc_id = Get_Loc_ID();
              if(loc_id >= 5){
                
                memset(strprint, 0, sizeof(strprint));
                make_coord_json_string(strprint, &fs, loc_id-4);
                printk("\n%s\n", strprint);
                err = publish_data(&client, "pubtopic/test", &strprint, sizeof(strprint), MQTT_QOS_0_AT_MOST_ONCE);
                if(err){
                    printk("Error publishing: %d", err);
                }

//                num_of_locs = (loc_id-1)/4;
//                make_coordinate_struct_array(&fs, coords_data);
//                make_coord_array_json_string(bigprint, coords_data, num_of_locs);
//                printk("%s\n", bigprint);

              }
        }


//---
        while(k_uptime_get()-loop_timer <= 20);
        loop_timer = k_uptime_get();
    }
}


void test_buttons_leds(){
    led_setup();
    button_setup();

    while(1){            
        if(button_is_pressed(1)){
            led_on(1);
        }else{
            led_off(1);
        }
        if(detect_press(1)){
            toggle_led(2);
        }
    }
}


int main(void){

    k_sleep(K_SECONDS(2));

    test_gps();

    return 0;
}

